#include <sp/spBaseLib.h>
#include <sp/spAudio.h>
#include <sp/spInputPlugin.h>

#include <sp/spMain.h>

static int debug_level;
static spBool help_flag;

static spBool supported_only;
static spBool display_song_info;

static double offset_s = 0.0;
static long buffer_length = 0;
static int num_buffer;

static char *driver_name = NULL;
static char *plugin_search_path = NULL;

static spOptions options;
static spOption option[] = {
    {"-o", "-offset", "offset time [s]", "offset",
	 SP_TYPE_DOUBLE, &offset_s, NULL},
    {"-song", NULL, "display song info", "display_song_info",
	 SP_TYPE_BOOLEAN, &display_song_info, SP_FALSE_STRING},
    {"-so", NULL, "play supported file only", "supported_only",
	 SP_TYPE_BOOLEAN, &supported_only, SP_FALSE_STRING},
    {"-buf", NULL, "buffer length for audio output", "buffer_length",
	 SP_TYPE_LONG, &buffer_length, "1024"},
    {"-nbuf", NULL, "number of buffers (valid only on Windows)", "num_buffer",
	 SP_TYPE_LONG, &num_buffer, "16"},
    {"-driver", NULL, "audio driver name", "driver_name",
	 SP_TYPE_STRING, &driver_name, NULL},
    {"-path", NULL, "extra plugin search path", "plugin_path",
	 SP_TYPE_STRING, &plugin_search_path, NULL},
    {"-debug", NULL, "debug level", NULL,
	 SP_TYPE_INT, &debug_level, NULL},
    {"-h", "-help", "display this message", NULL,
	 SP_TYPE_BOOLEAN, &help_flag, SP_FALSE_STRING},
    /* options for input_raw */
    SpNullOption("-f", "-freq", SP_TYPE_NONE),
    SpNullOption("-c", "-channel", SP_TYPE_NONE),
    SpNullOption("-b", "-bit", SP_TYPE_NONE),
    SpNullOption("-H", "-head", SP_TYPE_NONE),
    SpNullOption("-F", "-format", SP_TYPE_NONE),
};

static char *filelabel[] = {
    "<input file>",
};

int spMain(int argc, char *argv[])
{
    long i;
    spLong offset;
    char *input_file;
    long buffer_size;
    double *data;
    long data_length;
    long nread;
    spPlugin *i_plugin;
    spWaveInfo wave_info;
    spSongInfo song_info;
    spAudio audio;
	    
    /* get command line options */
    spSetHelpMessage(&help_flag, "test for playing sound file");
    options = spGetOptions(argc, argv, option, filelabel);
    spIgnoreUnknownOption(options, SP_TRUE);
    spGetOptionsValue(argc, argv, options);
    spSetDebugLevel(debug_level);
    spCheckNumFile(options);

    input_file = spGetFile(options);
    spDebug(1, "main", "input_file = %s\n", input_file);

    spInitWaveInfo(&wave_info);

    if (supported_only) {
	i_plugin = spOpenFilePluginArg(NULL, input_file, "r", SP_PLUGIN_DEVICE_FILE,
				       &wave_info, &song_info, argc, argv, NULL);
    } else {
	i_plugin = spOpenFilePluginArgAuto("input_raw", input_file, "r", SP_PLUGIN_DEVICE_FILE,
					   &wave_info, &song_info, argc, argv, NULL);
    }

    if (i_plugin != NULL) {
	if ((audio = spInitAudioDriver(driver_name)) == NULL) {
	    spWarning("Can't initialize audio.\n");
	} else {
	    spDebug(1, NULL, "init audio done\n");

	    if (display_song_info == SP_TRUE) {
		spShowSongInfo(&song_info, NULL, stdout);
	    }

	    buffer_size = buffer_length * (wave_info.samp_bit / 8);

	    spSetAudioSampleRate(audio, wave_info.samp_rate);
	    spSetAudioChannel(audio, wave_info.num_channel);
	    spSetAudioNumBuffer(audio, num_buffer);
	    spSetAudioBufferSize(audio, buffer_size);

	    /* open output audio device */
	    if (!spOpenAudioDevice(audio, "w")) {
		spWarning("Can't open audio device.\n");
	    } else {
		spDebug(1, NULL, "open output audio device done\n");

		/* create data */
		data_length = buffer_length;
		data = xalloc(data_length, double);

		offset = (spLong)spRound(offset_s * wave_info.samp_rate);
		if (offset > 0) {
		    spSeekPlugin(i_plugin, offset);
		}

		for (i = 0;; i++) {
		    if ((nread = spReadPluginDouble(i_plugin, data, data_length)) <= 0) {
			break;
		    }
		    spDebug(100, "main", "nread = %ld\n", nread);

		    spWriteAudioDouble(audio, data, nread);
		}

		xfree(data);
		spCloseAudioDevice(audio);
	    }
	    
	    spFreeAudioDriver(audio);
	}

	spCloseFilePlugin(i_plugin);
    } else {
	spError(1, "Cannot open file: %s\n", input_file);
    }
    
    return 0;
}
