/*
 *	plugin_ogg.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sp/spDefs.h>
#include <sp/spBase.h>
#include <sp/spFile.h>
#include <sp/spMemory.h>

#include <sp/spInputPluginP.h>

typedef struct _spOggPluginInstance
{
    OggVorbis_File vf;

    ogg_stream_state os;
    ogg_page og;
    ogg_packet op;
    vorbis_info vi;
    vorbis_comment vc;
    vorbis_dsp_state vd;
    vorbis_block vb;
    
    int num_channel;
    long samp_rate;

    int quality;
    long bit_rate;
    
    char filename[SP_MAX_PATHNAME];
    FILE *fp;
    spLong current_pos;
    spLong total_length;

    spSongInfoV2 song_info;

    char *buffer;
    long buffer_size;
    long buffer_nread;
    long buffer_copy_offset;
} *spOggPluginInstance;

typedef struct _spOggSongInfoTable {
    unsigned long mask;
    char *name;
    int offset;
    int size;
} spOggSongInfoTable;

static spOggSongInfoTable sp_ogg_song_info_table[] = {
    {SP_SONG_ARTIST_MASK, "ARTIST", spOffset(spSongInfoV2Ptr, artist), SP_SONG_INFO_SIZE},
    {SP_SONG_ALBUM_MASK, "ALBUM", spOffset(spSongInfoV2Ptr, album), SP_SONG_INFO_SIZE},
    {SP_SONG_TITLE_MASK, "TITLE", spOffset(spSongInfoV2Ptr, title), SP_SONG_INFO_SIZE},
    {SP_SONG_GENRE_MASK, "GENRE", spOffset(spSongInfoV2Ptr, genre), SP_SONG_INFO_GENRE_SIZE},
    {SP_SONG_COPYRIGHT_MASK, "COPYRIGHT", spOffset(spSongInfoV2Ptr, copyright), SP_SONG_INFO_SIZE},
    {SP_SONG_RELEASE_MASK, "DATE", spOffset(spSongInfoV2Ptr, release), SP_SONG_INFO_RELEASE_SIZE},
    {SP_SONG_ISRC_MASK, "ISRC", spOffset(spSongInfoV2Ptr, isrc), SP_SONG_INFO_ISRC_SIZE},
    {SP_SONG_COMMENT_MASK, "DESCRIPTION", spOffset(spSongInfoV2Ptr, comment), SP_SONG_INFO_SIZE},
    {SP_SONG_SOFTWARE_MASK, "ENCODER", spOffset(spSongInfoV2Ptr, software), SP_SONG_INFO_SOFTWARE_SIZE},
    {SP_SONG_SOFTWARE_MASK, "ENCODED BY", spOffset(spSongInfoV2Ptr, software), SP_SONG_INFO_SOFTWARE_SIZE},
    {0, NULL, 0, 0},
};

static spBool spInitPluginOgg(const char *lang)
{
    /*spSetDebugLevel(10);*/
    return SP_TRUE;
}

static spBool spFreePluginOgg(void)
{
    return SP_TRUE;
}

static void *spInitPluginInstanceOgg(const char *lang)
{
    spOggPluginInstance instance;
    
    instance = xalloc(1, struct _spOggPluginInstance);

    memset(&instance->vf, 0, sizeof(OggVorbis_File));
    
    instance->num_channel = 2;
    instance->samp_rate = 44100;
    
    instance->bit_rate = 0;
    instance->quality = 4;
    
    strcpy(instance->filename, "");
    instance->fp = NULL;
    instance->current_pos = 0;
    instance->total_length = 0;

    spInitSongInfoV2(&instance->song_info);
    
    return (void *)instance;
}

static spBool spFreePluginInstanceOgg(void *instance)
{
    xfree(instance);
    return SP_TRUE;
}

static spBool spSetPluginFileTypeOgg(void *instance, int index)
{
    return SP_TRUE;
}

static int spGetPluginFileTypeOgg(void *instance)
{
    return 0;
}

static spBool spSetPluginSongInfoOgg(void *instance, spSongInfo *song_info)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;
    
    spCopySongInfoV2(&pinstance->song_info, (spSongInfoV2 *)song_info);

    return SP_TRUE;
}

static spBool spGetPluginSongInfoOgg(void *instance, spSongInfo *song_info)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;

    spCopySongInfoV2((spSongInfoV2 *)song_info, &pinstance->song_info);

    return SP_TRUE;
}

static const char *spGetPluginBestSuffixOgg(void *instance)
{
    return ".ogg";
}

static spBool spSetPluginSampleBitOgg(void *instance, int samp_bit)
{
    if (samp_bit != 16) {
	return SP_FALSE;
    }

    return SP_TRUE;
}

static spBool spGetPluginSampleBitOgg(void *instance, int *samp_bit)
{
    *samp_bit = 16;

    return SP_TRUE;
}

static spBool spSetPluginChannelOgg(void *instance, int num_channel)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;

    pinstance->num_channel = num_channel;
    
    return SP_TRUE;
}

static spBool spGetPluginChannelOgg(void *instance, int *num_channel)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;
    
    *num_channel = pinstance->num_channel;

    return SP_TRUE;
}

static spBool spSetPluginSampleRateOgg(void *instance, double samp_rate)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;

    pinstance->samp_rate = (long)spRound(samp_rate);

    return SP_TRUE;
}

static spBool spGetPluginSampleRateOgg(void *instance, double *samp_rate)
{
    spOggPluginInstance pinstance = (spOggPluginInstance)instance;
    
    *samp_rate = (double)pinstance->samp_rate;

    return SP_TRUE;
}

static spBool spSetPluginOtherInfoOgg(void *instance, const char *id, char *data)
{
    
    return SP_FALSE;
}

static spBool spGetPluginOtherInfoOgg(void *instance, const char *id, char *data)
{

    return SP_FALSE;
}
